@@ -32,7 +32,15 @@ free_buffer(buffer *buf);
 inline PyObject *
 getPyString(buffer *buf);
 
+inline PyObject *
+getPyStringAndDecode(buffer *buf);
+
 inline char *
 getString(buffer *buf);
 
+inline void
+buffer_list_fill(void);
+
+inline void
+buffer_list_clear(void);
 #endif
