@@ -106,6 +106,32 @@ static PyObject *http_method_mkactivity;
 static PyObject *http_method_checkout;
 static PyObject *http_method_merge;
 
+static inline PyObject * 
+new_environ(client_t *client)
+{
+    register PyObject *object, *environ;
+
+    environ = PyDict_New();
+    PyDict_SetItem(environ, version_key, version_val);
+    PyDict_SetItem(environ, scheme_key, scheme_val);
+    PyDict_SetItem(environ, errors_key, errors_val);
+    PyDict_SetItem(environ, multithread_key, multithread_val);
+    PyDict_SetItem(environ, multiprocess_key, multiprocess_val);
+    PyDict_SetItem(environ, run_once_key, run_once_val);
+    PyDict_SetItem(environ, script_key, script_val);
+    PyDict_SetItem(environ, server_name_key, server_name_val);
+    PyDict_SetItem(environ, server_port_key, server_port_val);
+    PyDict_SetItem(environ, file_wrapper_key, file_wrapper_val);
+     
+    object = PyString_FromString(client->remote_addr);
+    PyDict_SetItem(environ, remote_addr_key, object);
+    Py_DECREF(object);
+
+    object = PyString_FromFormat("%d", client->remote_port);
+    PyDict_SetItem(environ, remote_port_key, object);
+    Py_DECREF(object);
+    return environ;
+}
 
 static inline void
 key_upper(char *s, const char *key, size_t len)
@@ -214,6 +240,21 @@ get_client(http_parser *p)
 int
 message_begin_cb(http_parser *p)
 {
+#ifdef DEBUG
+    printf("message_begin_cb \n");
+#endif
+
+    client_t *client = get_client(p);
+    
+    client->req = new_request();
+    client->environ = new_environ(client);
+    client->complete = 0;
+    client->bad_request_code = 0;
+    client->body_type = BODY_TYPE_NONE;
+    client->body_readed = 0;
+    client->body_length = 0;
+    client->req->env = client->environ;
+    push_request(client->request_queue, client->req);
     return 0;
 }
 
@@ -408,6 +449,9 @@ fragment_cb (http_parser *p, const char *buf, size_t len, char partial)
 int
 body_cb (http_parser *p, const char *buf, size_t len, char partial)
 {
+#ifdef DEBUG
+    printf("body_cb \n");
+#endif
     client_t *client = get_client(p);
     if(max_content_length <= client->body_readed + len){
 
@@ -464,6 +508,8 @@ headers_complete_cb (http_parser *p)
         client->bad_request_code = 413;
         return -1;
     }
+    
+
     if(p->http_major == 1 && p->http_minor == 1){
         obj = server_protocol_val11;
     }else{
@@ -573,7 +619,7 @@ headers_complete_cb (http_parser *p)
     
     PyDict_SetItem(env, request_method_key, obj);
 
-    PyMem_Free(req);
+    //free_request(req);
     client->req = NULL;
     client->body_length = p->content_length;
     
@@ -582,14 +628,25 @@ headers_complete_cb (http_parser *p)
     PyDict_SetItem(env, client_key, obj);
     Py_DECREF(obj);
 
+#ifdef DEBUG
+    printf("headers_complete_cb \n");
+#endif
     return 0;
 }
 
 int
 message_complete_cb (http_parser *p)
 {
+#ifdef DEBUG
+    printf("message_complete_cb \n");
+#endif
     client_t *client = get_client(p);
     client->complete = 1;
+    
+    request *req = client->request_queue->tail;
+    req->body = client->body;
+    req->body_type = client->body_type;
+    
     return 0;
 }
 
@@ -606,41 +663,15 @@ static http_parser_settings settings =
   ,.on_message_complete = message_complete_cb
   };
 
+
 static PyMethodDef method = {"file_wrapper", (PyCFunction)file_wrapper, METH_VARARGS, 0};
 
 inline int
 init_parser(client_t *cli, const char *name, const short port)
 {
-    register PyObject *object;
 
     cli->http = (http_parser*)PyMem_Malloc(sizeof(http_parser));
     memset(cli->http, 0, sizeof(http_parser));
-    
-    cli->environ = PyDict_New();
-    if (cli->environ == NULL)
-    {
-        return -1;
-    }
-    
-    PyDict_SetItem(cli->environ, version_key, version_val);
-    PyDict_SetItem(cli->environ, scheme_key, scheme_val);
-    PyDict_SetItem(cli->environ, errors_key, errors_val);
-    PyDict_SetItem(cli->environ, multithread_key, multithread_val);
-    PyDict_SetItem(cli->environ, multiprocess_key, multiprocess_val);
-    PyDict_SetItem(cli->environ, run_once_key, run_once_val);
-    PyDict_SetItem(cli->environ, script_key, script_val);
-    PyDict_SetItem(cli->environ, server_name_key, server_name_val);
-    PyDict_SetItem(cli->environ, server_port_key, server_port_val);
-    PyDict_SetItem(cli->environ, file_wrapper_key, file_wrapper_val);
-     
-    object = PyString_FromString(cli->remote_addr);
-    PyDict_SetItem(cli->environ, remote_addr_key, object);
-    Py_DECREF(object);
-
-    object = PyString_FromFormat("%d", cli->remote_port);
-    PyDict_SetItem(cli->environ, remote_port_key, object);
-    Py_DECREF(object);
-    
 
     http_parser_init(cli->http, HTTP_REQUEST);
     cli->http->data = cli;
@@ -653,9 +684,6 @@ execute_parse(client_t *cli, const char *data, size_t len)
     size_t ret = http_parser_execute(cli->http, &settings, data, len);
     //check new protocol
     cli->upgrade = cli->http->upgrade;
-    
-    cli->http_major = cli->http->http_major;
-    cli->http_minor = cli->http->http_minor;
 
     return ret;
 }
