@@ -11,10 +11,14 @@
 
 #define MSG_400 ("HTTP/1.0 400 Bad Request\r\nContent-Type: text/html\r\nServer: " SERVER "\r\n\r\n<html><head><title>Bad Request</head><body><p>Bad Request.</p></body></html>")
 
+#define MSG_408 ("HTTP/1.0 408 Request Timeout\r\nContent-Type: text/html\r\nServer: " SERVER "\r\n\r\n<html><head><title>Request Timeout</head><body><p>Request Timeout.</p></body></html>")
+
 #define MSG_411 ("HTTP/1.0 411 Length Required\r\nContent-Type: text/html\r\nServer: " SERVER "\r\n\r\n<html><head><title>Length Required</head><body><p>Length Required.</p></body></html>")
 
 #define MSG_413 ("HTTP/1.0 413 Request Entity Too Large\r\nContent-Type: text/html\r\nServer: " SERVER "\r\n\r\n<html><head><title>Request Entity Too Large</head><body><p>Request Entity Too Large.</p></body></html>")
 
+#define MSG_417 ("HTTP/1.1 417 Expectation Failed\r\nContent-Type: text/html\r\nServer: " SERVER "\r\n\r\n<html><head><title>Expectation Failed</head><body><p>Expectation Failed.</p></body></html>")
+
 ResponseObject *start_response = NULL;
 
 static inline int 
@@ -83,12 +87,18 @@ send_error_page(client_t *client)
         case 400:
             blocking_write(client, MSG_400, sizeof(MSG_400) -1);
             break;
+        case 408:
+            blocking_write(client, MSG_400, sizeof(MSG_408) -1);
+            break;
         case 411:
             blocking_write(client, MSG_411, sizeof(MSG_411) -1);
             break;
         case 413:
             blocking_write(client, MSG_413, sizeof(MSG_413) -1);
             break;
+        case 417:
+            blocking_write(client, MSG_417, sizeof(MSG_417) -1);
+            break;
         case 503:
             blocking_write(client, MSG_503, sizeof(MSG_503) -1);
             break;
@@ -135,6 +145,22 @@ set2bucket(write_bucket *bucket, char *buf, size_t len)
     bucket->total_size += len;
 }
 
+static inline void
+set_chunked_data(write_bucket *bucket, char *lendata, size_t lenlen, char *data, size_t datalen)
+{
+    set2bucket(bucket, lendata, lenlen);
+    set2bucket(bucket, CRLF, 2);
+    set2bucket(bucket, data, datalen);
+    set2bucket(bucket, CRLF, 2);
+}
+
+static inline void
+set_last_chunked_data(write_bucket *bucket)
+{
+    set2bucket(bucket, "0", 1);
+    set2bucket(bucket, CRLF, 2);
+}
+
 
 static inline void
 add_header(write_bucket *bucket, char *key, size_t keylen, char *val, size_t vallen)
@@ -146,9 +172,6 @@ add_header(write_bucket *bucket, char *key, size_t keylen, char *val, size_t val
 }
 
 
-static inline int 
-writev_bucket(write_bucket *data);
-
 static inline int 
 writev_bucket(write_bucket *data)
 {
@@ -167,6 +190,7 @@ writev_bucket(write_bucket *data)
             return -1;
         }
     }if(w == 0){
+        data->sended = 1;
         return 1;
     }else{
         if(data->total > w){
@@ -189,7 +213,9 @@ writev_bucket(write_bucket *data)
             // again later
             return writev_bucket(data);
         }
+        data->sended = 1;
     }
+    data->sended = 1;
     return 1;
 }
 
@@ -251,7 +277,7 @@ write_headers(client_t *client, char *data, size_t datalen)
         hlen = PySequence_Fast_GET_SIZE(headers);
         Py_DECREF(headers);
     }
-    bucket = new_write_bucket(client->fd, ( hlen * 4) + 32 );
+    bucket = new_write_bucket(client->fd, (hlen * 4) + 40 );
     
     object = client->http_status;
     if(object){
@@ -263,9 +289,6 @@ write_headers(client_t *client, char *data, size_t datalen)
         add_header(bucket, "Server", 6,  SERVER, sizeof(SERVER) -1);
         cache_time_update();
         add_header(bucket, "Date", 4, (char *)http_time, 29);
-        if(client->keep_alive == 1){
-            add_header(bucket, "Connection", 10, "keep-alive", 10);
-        }
     }
 
     //write header
@@ -354,14 +377,33 @@ write_headers(client_t *client, char *data, size_t datalen)
     //header done 
     
     // check content_length_set
-    // set content_length
-    //if(data){
-    //    set_content_length(client, bucket, data, datalen); 
-    //}
+    if(!client->content_length_set && client->http->http_minor == 1){
+        //Transfer-Encoding chunked
+        add_header(bucket, "Transfer-Encoding", 17, "chunked", 7);
+        client->chunked_response = 1;
+    }
+
+    if(client->keep_alive == 1){
+        //Keep-Alive
+        add_header(bucket, "Connection", 10, "Keep-Alive", 10);
+    }else{
+        add_header(bucket, "Connection", 10, "close", 5);
+    }
 
     set2bucket(bucket, CRLF, 2);
+    
     if(data){
-        set2bucket(bucket, data, datalen);
+        if(client->chunked_response){
+            char lendata[32];
+            int i = 0;
+            i = snprintf(lendata, 32, "%zx", datalen);
+#ifdef DEBUG
+            printf("Transfer-Encoding chunk_size %s \n", lendata);
+#endif
+            set_chunked_data(bucket, lendata, i, data, datalen);  
+        }else{
+            set2bucket(bucket, data, datalen);
+        }
     }
     client->bucket = bucket;
     int ret = writev_bucket(bucket);
@@ -393,6 +435,9 @@ write_sendfile(int out_fd, int in_fd, size_t count)
 
     if (size == 0) {
         struct stat info;
+#ifdef DEBUG
+        printf("call fstat \n");
+#endif
         if (fstat(in_fd, &info) == -1){
             PyErr_SetFromErrno(PyExc_IOError);
             write_error_log(__FILE__, __LINE__); 
@@ -401,7 +446,7 @@ write_sendfile(int out_fd, int in_fd, size_t count)
 
         size = info.st_size - lseek(in_fd, 0, SEEK_CUR);
     }
-    return sendfile(out_fd, in_fd, NULL, count);
+    return sendfile(out_fd, in_fd, NULL, size);
 }
 
 inline void 
@@ -498,13 +543,28 @@ processs_write(register client_t *client)
             if(PyString_Check(item)){
                 PyString_AsStringAndSize(item, &buf, &buflen);
                 //write
-                bucket = new_write_bucket(client->fd, 1);
-                set2bucket(bucket, buf, buflen);
+                if(client->chunked_response){
+                    bucket = new_write_bucket(client->fd, 4);
+                    
+                    char lendata[32];
+                    int i = 0;
+                    i = snprintf(lendata, 32, "%zx", buflen);
+#ifdef DEBUG
+                    printf("Transfer-Encoding chunk_size %s \n", lendata);
+#endif
+                    set_chunked_data(bucket, lendata, i, buf, buflen);  
+                }else{
+                    bucket = new_write_bucket(client->fd, 1);
+                    set2bucket(bucket, buf, buflen);
+                }
                 ret = writev_bucket(bucket);
                 if(ret <= 0){
+                    client->bucket = bucket;
                     Py_DECREF(item);
                     return ret;
                 }
+
+                free_write_bucket(bucket);
                 //mark
                 client->write_bytes += buflen;
                 //check write_bytes/content_length
@@ -525,6 +585,13 @@ processs_write(register client_t *client)
             }
             Py_DECREF(item);
         }
+
+        if(client->chunked_response){
+            bucket = new_write_bucket(client->fd, 2);
+            set_last_chunked_data(bucket);
+            writev_bucket(bucket);
+            free_write_bucket(bucket);
+        }
         close_response(client);
     }
     return 1;
@@ -589,6 +656,7 @@ start_response_file(client_t *client)
     size = info.st_size;
     client->content_length_set = 1;
     client->content_length = size;
+    
     return write_headers(client, NULL, 0);
 }
 
@@ -682,15 +750,6 @@ clear_start_response(void)
 inline PyObject *  
 create_start_response(client_t *cli)
 {
-    /*
-    if(start_response){
-        start_response->cli = cli;
-        return (PyObject *)start_response;
-    }
-    start_response = PyObject_NEW(ResponseObject, &ResponseObjectType);
-    if(start_response == NULL)
-        return NULL;
-    */
     start_response->cli = cli;
     return (PyObject *)start_response;
 }
@@ -778,8 +837,12 @@ ResponseObject_call(PyObject *obj, PyObject *args, PyObject *kw)
     Py_INCREF(self->cli->headers);
     
     Py_XDECREF(self->cli->http_status);
-    //self->cli->http_status =  PyString_FromFormat("HTTP/1.1 %s\r\n", PyString_AS_STRING(status));
-    self->cli->http_status =  PyString_FromFormat("HTTP/1.0 %s\r\n", PyString_AS_STRING(status));
+
+    if(self->cli->http->http_minor == 1){
+        self->cli->http_status =  PyString_FromFormat("HTTP/1.1 %s\r\n", PyString_AS_STRING(status));
+    }else{
+        self->cli->http_status =  PyString_FromFormat("HTTP/1.0 %s\r\n", PyString_AS_STRING(status));
+    }
 
     Py_RETURN_NONE;
 }
